import 'package:flutter/material.dart';
import 'package:loyalty_foop/bloc/common_widgets/postcardmedia.dart';

import 'package:loyalty_foop/models/welcomeMessagesListResponse.dart';
import 'package:loyalty_foop/utils/colors.dart';
import 'package:loyalty_foop/utils/customcard.dart';
import 'package:loyalty_foop/utils/hexColors.dart';
import 'package:loyalty_foop/utils/localization.dart';
import 'package:loyalty_foop/utils/locator.dart';
import 'package:loyalty_foop/utils/text_styling.dart';

import 'PostCardFooter.dart';

class RewardsWidget extends StatefulWidget {
  final WelcomeMessageItem item;
  final Function(WelcomeMessageItem? item) editCallBack;
  final Function(int ?id) deleteCallBack;

  const RewardsWidget(
      {Key? key,
        required this.item,
        required this.editCallBack,
        required this.deleteCallBack})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => RewardsWidgetState();
}

class RewardsWidgetState extends State<RewardsWidget> {

  AppLocalizations localizations  = locatorRewards<AppLocalizations>();
  Widget _simplePopup() {
    // var name = headerData.title;
    return PopupMenuButton<String>(
      padding: EdgeInsets.only(right: 0),
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(4)),
      itemBuilder: (context) => getItems(),
      onSelected: (value) {
        switch (value) {
          case 'delete':
            {
              widget.deleteCallBack(widget.item.id);

              break;
            }
          case 'edit':
            {
              widget.editCallBack(widget.item);
              break;
            }
          case 'hide':
            {
              break;
            }
          case 'statistics':
            {
              break;
            }

          case 'unfollow':
            {
              break;
            }
          case 'block':
            {
              break;
            }
          case 'topic':
            {
              break;
            }
          case 'report':
            {
              break;
            }
        }
      },
      icon: Icon(
        Icons.more_vert,
        color: HexColor(AppColors.appColorBlack65),
      ),
    );
  }

  late TextStyleElements styleElements;

  @override
  Widget build(BuildContext context) {
    styleElements = TextStyleElements(context);

    return appCard(
      padding: EdgeInsets.all(0),
      child: Column(
        children: [

          Visibility(
            visible: widget.item.messageDetails!=null && widget.item.messageDetails?.mediaDetails!=null &&widget.item.messageDetails!.mediaDetails!.isNotEmpty,
            child: PostCardMedia(
                postType: "feed",
                mediaList:widget.item.messageDetails!.mediaDetails!.isNotEmpty? widget.item.messageDetails?.mediaDetails:[],
                pagePosition: 0,
                onItemClick: () {}),
          ),
          Padding(
            padding:
            EdgeInsets.only(left: 8, right: 8.0, top: 8.0, bottom: 16.0),
            child: PostCardFooter(
              postOwnerTypeId: widget.item.businessId,
              postId: widget.item.id,
              isRewards: true,
              postType: "feed",
              isWelcomeChat: widget.item.messageType=="chat",
              welcomeMessageData: widget.item.messageDetails?.contentMeta,
            ),
          )
        ],
      ),
    );
  }

  List<PopupMenuEntry<String>> getItems() {
    List<PopupMenuEntry<String>> popupmenuList = [];

    popupmenuList.add(
      PopupMenuItem(
        value: 'delete',
        child: Text(
          localizations.translate('delete'),
        ),
      ),
    );

    popupmenuList.add(
      PopupMenuItem(
        value: 'edit',
        child: Text(
          localizations.translate('edit'),
        ),
      ),
    );
    return popupmenuList;
  }
}
