import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:loyalty_foop/bloc/common_widgets/common_list_tile.dart';
import 'package:loyalty_foop/bloc/common_widgets/loading_indicator.dart';
import 'package:loyalty_foop/bloc/common_widgets/paginator_list_view.dart';
import 'package:loyalty_foop/bloc/cubits/cubit_main.dart';
import 'package:loyalty_foop/bloc/rewards_module/pages/set_loyalty_details.dart';
import 'package:loyalty_foop/bloc/states/main_state.dart';
import 'package:loyalty_foop/bloc/states/states.dart';
import 'package:loyalty_foop/models/LoyaltyTypeList.dart';
import 'package:loyalty_foop/utils/app_buttons.dart';
import 'package:loyalty_foop/utils/basicInfo.dart';
import 'package:loyalty_foop/utils/colors.dart';
import 'package:loyalty_foop/utils/common_button.dart';
import 'package:loyalty_foop/utils/custom_app_bar.dart';
import 'package:loyalty_foop/utils/customcard.dart';
import 'package:loyalty_foop/utils/hexColors.dart';
import 'package:loyalty_foop/utils/localization.dart';
import 'package:loyalty_foop/utils/locator.dart';
import 'package:loyalty_foop/utils/searchBox.dart';
import 'package:loyalty_foop/utils/text_styling.dart';
import 'package:shared_preferences/shared_preferences.dart';


class LoyaltyTypePageNew extends StatefulWidget {
  @override
  LoyaltyTypePageNewState createState() => LoyaltyTypePageNewState();
}

class LoyaltyTypePageNewState extends State<LoyaltyTypePageNew> {
  final scrollController = ScrollController();
  late TextStyleElements styleElements;
  List<dynamic> list = [];
  bool isLastPage = false;
  String? searchValue;
  LoyaltyTypeItem? selectedItem;
  BasicInfo? basicInfo = BasicInfo();
  AppLocalizations? localizations = locatorRewards<AppLocalizations>();

  SharedPreferences? prefs = locatorRewards<SharedPreferences>();
  late BuildContext ctx;

  void setupScrollController() {
    scrollController.addListener(() {
      if (scrollController.position.atEdge) {
        if (scrollController.position.pixels != 0) {
          if (!isLastPage)
            BlocProvider.of<CubitMain>(ctx).loadPage(
                getBody(), ctx, basicInfo!.LOYALTY_TYPE_LIST, getData);
        }
      }
    });
  }

  dynamic getData(dynamic data) {
    return LoyaltyTypeList.fromJson(data);
  }

  String getBody() {
    return jsonEncode(
        {
          "entity_id": basicInfo!.businessId,
          "search_val": searchValue,
          "page_size": 20,
          "page_number": BlocProvider
              .of<CubitMain>(ctx)
              .page,
          "app_type": "FOOPWORKS"
        }
    );
  }

  @override
  void initState() {
    setupScrollController();
    WidgetsBinding.instance!.addPostFrameCallback((_) {
      BlocProvider.of<CubitMain>(context).loadPage(
          getBody(), context, basicInfo!.LOYALTY_TYPE_LIST, getData);
    });

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    this.ctx = context;
    styleElements = TextStyleElements(context);


    return Scaffold(
      appBar: appAppBar().getCustomAppBar(
        context,
        appBarTitle: localizations!.translate('select_l_type'),
        actions: [

          appTextButton(
            onPressed: () {
              Navigator.push(context,
                  MaterialPageRoute(builder: (BuildContext context) {
                    return SetLoyaltyDetails(

                    );
                  })).then((value) =>
              {
              });
            },
            child: Wrap(
              direction: Axis.horizontal,
              crossAxisAlignment: WrapCrossAlignment.center,
              children: [
                const Visibility(
                  visible: false,
                  child: Padding(
                    padding: EdgeInsets.only(right: 8.0),
                    child: SizedBox(
                      height: 15,
                      width: 15,
                      child: CircularProgressIndicator(),
                    ),
                  ),
                ),
                Text(
                  localizations!.translate("next"), style: styleElements
                    .headline6ThemeScalableW(context)
                    .copyWith(color: HexColor(AppColors.blueApp)),
                ),
                Icon(Icons.keyboard_arrow_right,
                    color: HexColor(AppColors.blueApp))
              ],
            ),
            shape: CircleBorder(),
          ),
        ],
        onBackButtonPress: () {
          Navigator.of(context).pop();
        },
      ),
      body: Stack(
        children: <Widget>[
          Container(
              child: NestedScrollView(
                  headerSliverBuilder: (BuildContext context,
                      bool innerBoxIsScrolled) {
                    return [
                      SliverToBoxAdapter(
                        child: SearchBox(
                          onvalueChanged: (s) {
                            searchValue = s;
                            BlocProvider
                                .of<CubitMain>(ctx)
                                .page = 1;
                            s.isNotEmpty ?
                            BlocProvider.of<CubitMain>(ctx).search(
                                getBody(), ctx, basicInfo!.LOYALTY_TYPE_LIST,
                                getData) : BlocProvider.of<CubitMain>(ctx)
                                .loadPage(
                                getBody(), ctx, basicInfo!.LOYALTY_TYPE_LIST,
                                getData);
                          },
                          hintText:localizations!.translate(
                              'search'),
                        ),
                      ),
                    ];
                  },
                  body: _getBody()
              )),
          Align(
              alignment: Alignment.bottomCenter,
              child: Container(
                height: 60,
                color: HexColor(AppColors.appColorWhite),
                child: Align(
                    alignment: Alignment.centerRight,
                    child: Container(
                      margin: const EdgeInsets.only(left: 16.0, right: 16.0),
                      child:
                      AppButtonCommon(
                        shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(18.0),
                            side: BorderSide(
                                color: HexColor(AppColors.appMainColor))),
                        onPressed: () async {
                          if (selectedItem != null)
                            Navigator.of(context)
                                .pop({'result': selectedItem});
                        },
                        color: HexColor(AppColors.appColorWhite),
                        child: Text(
                         localizations!.translate('next'),
                          style: styleElements
                              .headline6ThemeScalableW(context)
                              .copyWith(color: HexColor(AppColors.blueApp)),
                        ),
                      ),


                    )),
              ))
        ],
      )
      ,
    );
  }

  Widget _getBody() {
    return BlocBuilder<CubitMain, MainState>(builder: (context, state) {
      if (state is LoadingStatePagination && state.isFirstFetch) {
        return LoadingIndicator();
      }
      bool isLoading = false;
      if (state is LoadingStatePagination) {
        list = state.list;
        isLoading = true;
      }
      else if (state is Loaded) {
        list = state.list;
        isLastPage = state.isLastPage;
        selectedItem = state.selectedItem;
      }

      return
        appCard(
            padding: EdgeInsets.only(left: 0, right: 0, bottom: 60, top: 0),
            child:
            Padding(
              padding: const EdgeInsets.only(top: 8.0),
              child: PaginatorListView(
                scrollController: scrollController,
                list: list,
                isLoading: isLoading,
                itemBuilder: listItemBuilder,
              ),
            )

        );
    });
  }


  Widget listItemBuilder(value, int index) {
    LoyaltyTypeItem item = value;
    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () {

      },
      child: Padding(
          padding: const EdgeInsets.all(0),
          child: CommonListTile(
            title: item.loyaltyTypeName ?? "-",
            subTitle: item.loyaltyTypeDescription ?? "-",
            trailWidget: Checkbox(onChanged: (bool? value) {
              BlocProvider.of<CubitMain>(ctx).updateSelectedItem(
                  value ?? false, index);
            }, value: item.isSelected,),
          )
      ),);
  }
}